<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00004109NegociacaoContratoFactory {
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Insere os dados de Negociação de Contrato
    **/
    public function saveDadosNegContrato($params) {
        $sql = "INSERT INTO FCT_NEGCONTRATO
                       (NRNEGCONTRATO, NRPARCNEGOCIO, NRAGRUPAMENTO, IDTPFATURA, CDFILIAL, NRPARCFATURA, CDTIPOOPERFATURA, 
                		CDSERIENF, CDEMPRESA, CDINSCESTA, CDINSCMUNI, NRSEQCONTRATO, NRITEMOCONTRATO, NRITEMELEMENTO, 
                		DSVARIAVELNEGOCIA, IDTIPONEGOCIA, DSOBSNEGOCIA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, 
                		NRORGINCLUSAO, CDOPERSOLICINEGOCIA, NRMOTIVOPARALISACAO)
                VALUES 
                       (:NRNEGCONTRATO, :NRPARCNEGOCIO, :NRAGRUPAMENTO, :IDTPFATURA, :CDFILIAL, :NRPARCFATURA, :CDTIPOOPERFATURA, 
                		:CDSERIENF, :CDEMPRESA, :CDINSCESTA, :CDINSCMUNI, :NRSEQCONTRATO, :NRITEMOCONTRATO, :NRITEMELEMENTO, 
                		:DSVARIAVELNEGOCIA, :IDTIPONEGOCIA, :DSOBSNEGOCIA, :IDATIVO, :NRORG, SYSDATE, :OPERADOR, 
                		:NRORG, :CDOPERSOLICINEGOCIA, :NRMOTIVOPARALISACAO)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    /**
     * Insere as parcelas de Negociação
    **/
    public function saveParcelasNegContrato($params) {
        $sql = "INSERT INTO FCT_PARCELASNEGCONTR
                       (NRPARCELASNEGCONTR, NRNEGCONTRATO, DTINIPARCNEGOCIA, DTFIMPARCNEGOCIA, IDMULTIPLICAQTMED, VRPARCELANEGOCIA, 
                       IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES
                      (:NRPARCELASNEGCONTR, :NRNEGCONTRATO, :DTINIPARCNEGOCIA, :DTFIMPARCNEGOCIA, :IDMULTIPLICAQTMED, :VRPARCELANEGOCIA, 
                       :IDATIVO, :NRORG, SYSDATE, :OPERADOR, :NRORG)";
                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    /**
     * Editar os dados das parcelas de Negociação de Contrato
    **/
    public function editParcelasNegoContr($params) {
        $sql = "UPDATE FCT_PARCELASNEGCONTR
                   SET DTINIPARCNEGOCIA = :DTINIPARCNEGOCIA,
                	   DTFIMPARCNEGOCIA = :DTFIMPARCNEGOCIA,
                	   IDMULTIPLICAQTMED = :IDMULTIPLICAQTMED,
                	   VRPARCELANEGOCIA = :VRPARCELANEGOCIA,
                	   DTULTATU = SYSDATE,
                	   NRORGULTATU = :NRORG,
                	   CDOPERULTATU = :OPERADOR
                 WHERE NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR
                   AND NRNEGCONTRATO = :NRNEGCONTRATO
                   AND NRORG = :NRORG";
                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /**
     * Recebe a menor data inicial e a maior data final das parcelas de Negociação de Contrato
    **/
    public function recebeMinMaxParcelas($params) {
        $sql = "SELECT MIN(DTINIPARCNEGOCIA) DTMINPARCELAS, MAX(DTFIMPARCNEGOCIA) DTMAXPARCELAS
                  FROM FCT_PARCELASNEGCONTR 
                 WHERE NRNEGCONTRATO = :NRNEGCONTRATO 
                   AND NRORG = :NRORG ";
                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Deleta as parcelas de negociação de contrato*/
    public function deleteParcelasNegContrato ($params) {
        $sql = "DELETE FROM FCT_PARCELASNEGCONTR 
                 WHERE NRNEGCONTRATO = :NRNEGCONTRATO 
                   AND (DTINIPARCNEGOCIA BETWEEN :DTINIPARCNEGOCIA AND :DTFIMPARCNEGOCIA)
                   AND (DTFIMPARCNEGOCIA BETWEEN :DTINIPARCNEGOCIA AND :DTFIMPARCNEGOCIA)
                   AND NRORG = :NRORG";
                                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Verifica se existe uma parcela para o periodo informado
    **/
    public function validaParcelasPeriodo($params) {
        $sql = "SELECT COUNT (*) AS VALIDA
                  FROM FCT_PARCELASNEGCONTR 
                 WHERE NRNEGCONTRATO = :NRNEGCONTRATO 
                   AND (DTINIPARCNEGOCIA BETWEEN :DTINIPARCNEGOCIA AND :DTFIMPARCNEGOCIA)
                   AND (DTFIMPARCNEGOCIA BETWEEN :DTINIPARCNEGOCIA AND :DTFIMPARCNEGOCIA)
                   AND NRORG = :NRORG";
                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["VALIDA"];
    }
       
    /* Exclui a parcela de negociação de contrato*/
    public function excluirParcela ($params) {
        $sql = "DELETE FROM FCT_PARCELASNEGCONTR 
                 WHERE NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR 
                   AND NRNEGCONTRATO = :NRNEGCONTRATO 
                   AND NRORG = :NRORG";
                                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
       
    /**
     * Valida se ja existe um registro de negociação com os mesmos dados da que esta sendo inserida.
    **/
    public function NegociacoesParceiro($params) {
        $sql = "SELECT NRNEGCONTRATO, NRPARCNEGOCIO, NRAGRUPAMENTO, NRSEQCONTRATO, 
                       NRITEMOCONTRATO, NRITEMELEMENTO
                  FROM FCT_NEGCONTRATO
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG = :NRORG";
                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }    
    
    /**
     * Recebe a menor data inicial e a maior data final das parcelas de Negociação de Contrato
    **/
    public function recebeMinMaxParcelasNeg($params) {
        $sql = "SELECT TO_CHAR(MIN(DTINIPARCNEGOCIA), 'DD/MM/YYYY') DTMINPARCELAS, TO_CHAR(MAX(DTFIMPARCNEGOCIA), 'DD/MM/YYYY') DTMAXPARCELAS
                  FROM FCT_PARCELASNEGCONTR 
                 WHERE NRNEGCONTRATO = :NRNEGCONTRATO 
                   AND NRORG = :NRORG";
                        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getDadosNegociacao(array $params) {
        $sql = "SELECT NRNEGCONTRATO,
                       NRPARCNEGOCIO,
                       NRAGRUPAMENTO,
                       IDTPFATURA,
                       CDFILIAL,
                       NRPARCFATURA,
                       CDTIPOOPERFATURA,
                       CDSERIENF,
                       CDEMPRESA,
                       CDINSCESTA,
                       CDINSCMUNI,
                       NRSEQCONTRATO,
                       NRITEMOCONTRATO,
                       DSVARIAVELNEGOCIA,
                       IDTIPONEGOCIA,
                       DSOBSNEGOCIA,
                       NRORG,
                       IDATIVO,
                       DTINCLUSAO,
                       DTULTATU,
                       NRORGINCLUSAO,
                       CDOPERINCLUSAO,
                       NRORGULTATU,
                       CDOPERULTATU,
                       NRITEMELEMENTO
                  FROM FCT_NEGCONTRATO
                 WHERE NRNEGCONTRATO = :NRNEGCONTRATO 
                   AND NRORG         = :NRORG";
                        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getNegociacaoRepetida(array $params) {
        $sql = "SELECT PARC.NRPARCELASNEGCONTR
                  FROM FCT_PARCELASNEGCONTR PARC
                 INNER JOIN FCT_NEGCONTRATO NEG
                    ON PARC.NRNEGCONTRATO = NEG.NRNEGCONTRATO
                 WHERE ( :DTINI BETWEEN TRUNC(PARC.DTINIPARCNEGOCIA) AND TRUNC(PARC.DTFIMPARCNEGOCIA)
                        OR :DTFIN BETWEEN TRUNC(PARC.DTINIPARCNEGOCIA) AND TRUNC(PARC.DTFIMPARCNEGOCIA) )
                   AND NEG.NRPARCNEGOCIO    = :NRPARCNEGOCIO
                   AND NEG.NRAGRUPAMENTO    = :NRAGRUPAMENTO
                   AND NEG.CDTIPOOPERFATURA = :CDTIPOOPERFATURA
                   AND NEG.CDSERIENF        = :CDSERIENF
                   AND NEG.CDEMPRESA        = :CDEMPRESA
                   AND NEG.CDINSCESTA       = :CDINSCESTA
                   AND NEG.CDINSCMUNI       = :CDINSCMUNI
                   AND NEG.NRORG            = :NRORG
                   AND NEG.NRITEMOCONTRATO  IS NULL
                   AND NEG.IDATIVO          = 'S'
                   AND PARC.IDATIVO         = 'S'
                 UNION ALL
                SELECT PARC.NRPARCELASNEGCONTR
                  FROM FCT_PARCELASNEGCONTR PARC
                 INNER JOIN FCT_NEGCONTRATO NEG
                    ON PARC.NRNEGCONTRATO = NEG.NRNEGCONTRATO
                 WHERE ( :DTINI BETWEEN TRUNC(PARC.DTINIPARCNEGOCIA) AND TRUNC(PARC.DTFIMPARCNEGOCIA)
                        OR :DTFIN BETWEEN TRUNC(PARC.DTINIPARCNEGOCIA) AND TRUNC(PARC.DTFIMPARCNEGOCIA) )
                   AND NEG.NRPARCNEGOCIO    = :NRPARCNEGOCIO
                   AND NEG.NRSEQCONTRATO    = :NRSEQCONTRATO
                   AND NEG.NRITEMOCONTRATO  = :NRITEMOCONTRATO
                   AND NEG.NRORG            = :NRORG
                   AND NEG.IDATIVO          = 'S'
                   AND PARC.IDATIVO         = 'S'";
                        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /*public function getQtdRegistrosUtilizandoParcela(array $params) {
        $sql = "SELECT SUM(QTD) AS QTD
                  FROM (
                    SELECT COUNT(*) AS QTD
                      FROM FCT_MEDVARMEDICAOFATURA
                     WHERE NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR
                       AND NRORG              = :NRORG
                     UNION ALL
                    SELECT COUNT(*) AS QTD
                      FROM FCT_ITMEDICAOFATURA
                     WHERE NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR
                       AND NRORG              = :NRORG
                     UNION ALL
                    SELECT COUNT(*) AS QTD
                      FROM FCT_MEDICAOFATURA
                     WHERE NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR
                       AND NRORG              = :NRORG
                     UNION ALL
                    SELECT COUNT(*) AS QTD
                      FROM FCT_ITFATURACONT
                     WHERE NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR
                       AND NRORG              = :NRORG
                     UNION ALL
                    SELECT COUNT(*) AS QTD
                      FROM FCT_FATURACONT
                     WHERE NRPARCELASNEGCONTR = :NRPARCELASNEGCONTR
                       AND NRORG              = :NRORG
                )";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }*/
    
    public function getMedicoesComElemento($params){
        $sql = "SELECT MED.NRMEDICAOFATURA,
                MED.NRSEQCONTRATO,
                MED.NRPARCFATURA,
                MED.DTINICOMPET,
                MED.DTFINCOMPET,
                MED.IDSITUACAO AS IDSITUACAOMED,
                FAT.NRFATURACONT,
                FAT.IDSITUACAO AS IDSITUACAOFAT
            FROM FCT_MEDICAOFATURA MED
            LEFT JOIN FCT_RELACAGRUPCONTFAT RELAC
            ON MED.NRMEDICAOFATURA = RELAC.NRMEDICAOFATURA
            LEFT JOIN FCT_FATURACONT FAT
            ON RELAC.NRFATURACONT = FAT.NRFATURACONT
            WHERE ( :DTINICOMPET BETWEEN TRUNC(MED.DTINICOMPET) AND TRUNC(MED.DTFINCOMPET)
                OR :DTFINCOMPET BETWEEN TRUNC(MED.DTINICOMPET) AND TRUNC(MED.DTFINCOMPET) )
            AND MED.NRPARCFATURA   = :NRPARCNEGOCIO
            AND MED.NRSEQCONTRATO   = :NRSEQCONTRATO
            AND :NRITEMOCONTRATO IN (SELECT NRITEMOCONTRATO
                                       FROM FCT_ITMEDICAOFATURA
                                      WHERE NRMEDICAOFATURA = MED.NRMEDICAOFATURA)
            AND EXISTS (SELECT DISTINCT IDELEMENTORAIZ
                                   FROM FCT_MEDVARMEDICAOFATURA
                                  WHERE NRMEDICAOFATURA = MED.NRMEDICAOFATURA
                                    AND IDELEMENTORAIZ = (SELECT IDELEMENTO
                                                            FROM FCT_ITEMCONTRATO
                                                           WHERE NRITEMOCONTRATO = :NRITEMELEMENTO))
            AND MED.NRORG = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getMedicoesComServico($params){
        $sql = "SELECT MED.NRMEDICAOFATURA,
                MED.NRSEQCONTRATO,
                MED.NRPARCFATURA,
                MED.DTINICOMPET,
                MED.DTFINCOMPET,
                MED.IDSITUACAO AS IDSITUACAOMED,
                FAT.NRFATURACONT,
                FAT.IDSITUACAO AS IDSITUACAOFAT
            FROM FCT_MEDICAOFATURA MED
            LEFT JOIN FCT_RELACAGRUPCONTFAT RELAC
            ON MED.NRMEDICAOFATURA = RELAC.NRMEDICAOFATURA
            LEFT JOIN FCT_FATURACONT FAT
            ON RELAC.NRFATURACONT = FAT.NRFATURACONT
            WHERE ( :DTINICOMPET BETWEEN TRUNC(MED.DTINICOMPET) AND TRUNC(MED.DTFINCOMPET)
                OR :DTFINCOMPET BETWEEN TRUNC(MED.DTINICOMPET) AND TRUNC(MED.DTFINCOMPET) )
            AND MED.NRPARCFATURA   = :NRPARCNEGOCIO
            AND MED.NRSEQCONTRATO   = :NRSEQCONTRATO
            AND :NRITEMOCONTRATO IN (SELECT NRITEMOCONTRATO
                                       FROM FCT_ITMEDICAOFATURA
                                      WHERE NRMEDICAOFATURA = MED.NRMEDICAOFATURA)
            AND MED.NRORG = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getParcelasServico($params){
        $sql = "SELECT DISTINCT COUNT(*) AS QTD
                FROM FCT_FATURACONT FAT
                LEFT JOIN FCT_RELACAGRUPCONTFAT RELAC
                ON FAT.NRFATURACONT = RELAC.NRFATURACONT
                WHERE ( :DTINICOMPET BETWEEN TRUNC(FAT.DTINICOMPET) AND TRUNC(FAT.DTFINCOMPET)
                    OR :DTFINCOMPET BETWEEN TRUNC(FAT.DTINICOMPET) AND TRUNC(FAT.DTFINCOMPET) )
                AND FAT.NRPARCFATURA   = :NRPARCNEGOCIO
                AND RELAC.NRSEQCONTRATO   = :NRSEQCONTRATO
                AND :NRITEMOCONTRATO IN (SELECT NRITEMOCONTRATO
                                           FROM FCT_ITFATURACONT
                                          WHERE NRFATURACONT = FAT.NRFATURACONT)
                AND FAT.IDSITUACAO = '1'
                AND FAT.NRORG = :NRORG
             UNION ALL
             SELECT DISTINCT COUNT(*) AS QTD
                FROM FCT_FATURACONT FAT
                LEFT JOIN FCT_RELACAGRUPCONTFAT RELAC
                ON FAT.NRFATURACONT = RELAC.NRFATURACONT
                WHERE ( :DTINICOMPET BETWEEN TRUNC(FAT.DTINICOMPET) AND TRUNC(FAT.DTFINCOMPET)
                    OR :DTFINCOMPET BETWEEN TRUNC(FAT.DTINICOMPET) AND TRUNC(FAT.DTFINCOMPET) )
                AND FAT.NRPARCFATURA   = :NRPARCNEGOCIO
                AND RELAC.NRSEQCONTRATO   = :NRSEQCONTRATO
                AND :NRITEMOCONTRATO IN (SELECT NRITEMOCONTRATO
                                           FROM FCT_ITFATURACONT
                                          WHERE NRFATURACONT = FAT.NRFATURACONT)
                AND FAT.IDSITUACAO IN ('2', '3', '6')
                AND FAT.NRORG = :NRORG";
                    
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getParcelasElemento($params){
        $sql = "SELECT COUNT(*) AS QTD
                FROM FCT_FATURACONT FAT
                LEFT JOIN FCT_RELACAGRUPCONTFAT RELAC
                ON FAT.NRFATURACONT = RELAC.NRFATURACONT
                WHERE ( :DTINICOMPET BETWEEN TRUNC(FAT.DTINICOMPET) AND TRUNC(FAT.DTFINCOMPET)
                    OR :DTFINCOMPET BETWEEN TRUNC(FAT.DTINICOMPET) AND TRUNC(FAT.DTFINCOMPET) )
                AND FAT.NRPARCFATURA   = :NRPARCNEGOCIO
                AND RELAC.NRSEQCONTRATO   = :NRSEQCONTRATO
                AND :NRITEMOCONTRATO IN (SELECT NRITEMOCONTRATO
                                           FROM FCT_ITFATURACONT
                                          WHERE NRFATURACONT = FAT.NRFATURACONT)
                AND FAT.IDSITUACAO = '1'
                AND FAT.NRORG = :NRORG";
                    
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getMedicoesComAgrupamento($params){
        $sql = "SELECT FAT.NRFATURACONT,
                RELAC.NRSEQCONTRATO,
                FAT.NRPARCFATURA,
                FAT.DTINICOMPET,
                FAT.DTFINCOMPET,
                FAT.IDSITUACAO,
                RELAC.NRMEDICAOFATURA
            FROM FCT_FATURACONT FAT
            LEFT JOIN FCT_RELACAGRUPCONTFAT RELAC
            ON FAT.NRFATURACONT = RELAC.NRFATURACONT
            WHERE ( :DTINICOMPET BETWEEN TRUNC(FAT.DTINICOMPET) AND TRUNC(FAT.DTFINCOMPET)
                OR :DTFINCOMPET BETWEEN TRUNC(FAT.DTINICOMPET) AND TRUNC(FAT.DTFINCOMPET) )
            AND FAT.NRPARCFATURA = :NRPARCNEGOCIO
            AND RELAC.NRSEQCONTRATO IN (SELECT NRSEQCONTRATO
                                        FROM FCT_RELACAGRUPAMENTOCONT
                                       WHERE NRAGRUPAMENTO = :NRAGRUPAMENTO)
            AND FAT.NRORG = :NRORG";
                    
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
}

?>