<?php

namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\ApropriacaoRecursoFactory;
use Teknisa\Libs\Util\NewCode;

class ApropriacaoRecursoService {
    
    protected $apropriacaoRecursoFactory;
    protected $newcode;
   
    
    public function __construct(ApropriacaoRecursoFactory $apropriacaoRecursoFactory, NewCode $newcode) {
        $this->apropriacaoRecursoFactory = $apropriacaoRecursoFactory;
        $this->newCode                       = $newcode;
    }
    
    /**
     * Insere um registro de apropiação de recurso
     */
     
    public function inserirApropriacaoRecurso($nrrecursosagendcont, $nrparcnegrecurso, $dtapropriacao, $nrparcinterlocutor, $nrorg, $cdoperador, $iddiasemana, $iddiautil) {
        $params = array(
            "NRAPROPAGENDAMENTO"  => $this->newCode->getCodeByTableName("FCT_APROPAGENDAMENTO", $nrorg, 1, 10, true),
            "NRRECURSOSAGENDCONT" => $nrrecursosagendcont,
            "NRPARCNEGRECURSO"    => $nrparcnegrecurso,
            "IDSTATUSAPROPRIACAO" => "1",
            "DTAPROPRIACAO"       => $dtapropriacao,
            "QTHORASAPROPRIADAS"  => 0,
            "NRPARCINTERLOCUTOR"  => $nrparcinterlocutor,
            "NRORG"               => $nrorg,
            "IDATIVO"             => "S",
            "NRORGINCLUSAO"       => $nrorg,
            "CDOPERINCLUSAO"      => $cdoperador,
            "DTULTATU"            => null,
            "NRORGULTATU"         => null,
            "CDOPERULTATU"        => null,
            "IDDIASEMANA"         => $iddiasemana,
            "IDDIAUTIL"           => $iddiautil
        );
  
        $this->apropriacaoRecursoFactory->inserirApropriacaoRecurso($params);
    }
    
    /**
     * Deleta as apropriações do recurso da data informada em diante
     */
    public function deletarApropriacaoRecurso($nrrecursosagendcont, $dtapropriacao, $nrorg, $metodo, $diaSemana){
        $params = array(
            "NRRECURSOSAGENDCONT" => $nrrecursosagendcont,
            "DTAPROPRIACAO"       => $dtapropriacao,
            "NRORG"               => $nrorg
        );

        if($metodo == "EDIT")
            $this->apropriacaoRecursoFactory->deletarDiaApropriacaoRecurso($params);
        else
            $this->apropriacaoRecursoFactory->deletarApropriacaoRecurso($params, $diaSemana);
    }
    
    /**
     * Faz consuta  de validação de datas no banco 
     */
    public function recuperarParaValidacao($nrparcnegrecurso, $nrorg) {
        $params = array(
            "NRPARCNEGRECURSO" => $nrparcnegrecurso,
            "NRORG"          => $nrorg
        );
        
        return $this->apropriacaoRecursoFactory->recuperarParaValidacao($params);
    }
    
    /**
     * pesquisa apropiação com recurso e retona a quantidade 
     */
    
    public function recuperarApropriacaoComApontamento($nrrecursosagendcont, $nrorg, $dtapropriacao){
         $params = array(
            "NRRECURSOSAGENDCONT" => $nrrecursosagendcont,
            "NRORG" => $nrorg,
            "DTAPROPRIACAO" => $dtapropriacao
        );
        
        $resultadoConsulta = $this-> apropriacaoRecursoFactory-> recuperarApropriacaoComApontamento($params);
        return $resultadoConsulta[0]["QUANTIDADE"];
        
    }
    
    /**
     * Retorna as apropriações do recurso no intervalo de datas informado.
     */
    public function getApropriacoesRecursoPorPeriodo($nrrecursosagendcont, $dataInicial, $dataFinal, $nrorg) {
        $params = array(
            "NRRECURSOSAGENDCONT" => $nrrecursosagendcont,
            "NRORG"               => $nrorg,
            "DTINI"               => $dataInicial,
            "DTFIN"               => $dataFinal
        );
        
        return $this->apropriacaoRecursoFactory->getApropriacoesRecursoPorPeriodo($params);
    }
    
    /**
     * Insere um apontamento na apropriação do recurso.
     */
    public function inserirApontamento($nrapropagendamento, $idtipoapontamento, $idmotivosaida, $dthrinicioapont, $dthrfimcioapont, $qttempoapont, $dsobsapontamento, $nrorg, $cdoperador) {
        $novoCodigo = $this->newCode->getCodeByTableName("FCT_APONTAMENTOAPROPAGEN", $nrorg, 1, 10, true);
        
        $params = array(
            "NRAPONTAMENTOAPROPAGEN" => $novoCodigo,
            "NRAPROPAGENDAMENTO"     => $nrapropagendamento,
            "IDTIPOAPONTAMENTO"      => $idtipoapontamento,
            "IDMOTIVOSAIDA"          => $idmotivosaida,
            "DTHRINICIOAPONT"        => $dthrinicioapont,
            "DTHRFIMCIOAPONT"        => $dthrfimcioapont,
            "QTTEMPOAPONT"           => $qttempoapont,
            "DSOBSAPONTAMENTO"       => $dsobsapontamento,
            "NRORG"                  => $nrorg,
            "IDATIVO"                => "S",
            "NRORGINCLUSAO"          => $nrorg,
            "CDOPERINCLUSAO"         => $cdoperador
        );
        
        $this->apropriacaoRecursoFactory->inserirApontamento($params);
    }
}

